@@ -1,6 +1,10 @@
 # -*- coding: utf-8 -*-
 # @Author  : ydf
 # @Time    : 2019/8/8 0008 13:11
+"""
+所有中间件类型消费者的抽象基类。使实现不同中间件的消费者尽可能代码少。
+里面很多调用sys.stdout是由于print被全局打了猴子补丁变成可跳转。当识别到两个可跳转链接时候，第二个不能被识别，所以一些地方直接调用了sys.stdout。
+"""
 import abc
 import copy
 from multiprocessing import Process
@@ -27,7 +31,7 @@ from function_scheduling_distributed_framework.concurrent_pool.bounded_threadpoo
 from function_scheduling_distributed_framework.concurrent_pool.custom_evenlet_pool_executor import evenlet_timeout_deco, check_evenlet_monkey_patch, CustomEventletPoolExecutor
 from function_scheduling_distributed_framework.concurrent_pool.custom_gevent_pool_executor import gevent_timeout_deco, GeventPoolExecutor, check_gevent_monkey_patch
 from function_scheduling_distributed_framework.concurrent_pool.custom_threadpool_executor import CustomThreadPoolExecutor, check_not_monkey
-from function_scheduling_distributed_framework.consumers.redis_filter import RedisFilter
+from function_scheduling_distributed_framework.consumers.redis_filter import RedisFilter, RedisImpermanencyFilter
 from function_scheduling_distributed_framework.factories.publisher_factotry import get_publisher
 from function_scheduling_distributed_framework.utils import LoggerLevelSetterMixin, LogManager, decorators, nb_print, LoggerMixin, time_util, RedisMixin
 from function_scheduling_distributed_framework.utils.bulk_operation import MongoBulkWriteHelper, InsertOne
@@ -158,6 +162,8 @@ class ConsumersManager:
     @classmethod
     def show_all_consumer_info(cls):
         nb_print(f'当前解释器内，所有消费者的信息是：\n  {json.dumps(cls.consumers_queue__info_map, indent=4, ensure_ascii=False)}')
+        for _, consumer_info in cls.consumers_queue__info_map.items():
+            sys.stdout.write(f'{time.strftime("%H:%M:%S")} "{consumer_info["where_to_instantiate"]}"  \033[0;30;44m{consumer_info["queue_name"]} 的消费者\033[0m\n')
 
     @classmethod
     def get_concurrent_info(cls):
@@ -231,21 +237,23 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         """
         ConsumersManager.join_all_consumer_shedual_task_thread()
 
+    # noinspection PyProtectedMember
     def __init__(self, queue_name, *, consuming_function: Callable = None, function_timeout=0,
                  threads_num=50, specify_threadpool=None, concurrent_mode=1,
                  max_retry_times=3, log_level=10, is_print_detail_exception=True,
                  msg_schedule_time_intercal=0.0, qps=0, msg_expire_senconds=0,
                  logger_prefix='', create_logger_file=True, do_task_filtering=False,
-                 is_consuming_function_use_multi_params=True,
-                 is_do_not_run_by_specify_time_effect=False, do_not_run_by_specify_time=('10:00:00', '22:00:00'),
+                 task_filtering_expire_seconds=0, is_consuming_function_use_multi_params=True,
+                 is_do_not_run_by_specify_time_effect=False,
+                 do_not_run_by_specify_time=('10:00:00', '22:00:00'),
                  schedule_tasks_on_main_thread=False,
                  function_result_status_persistance_conf=FunctionResultStatusPersistanceConfig(False, False, 7 * 24 * 3600)):
         """
         :param queue_name:
         :param consuming_function: 处理消息的函数。
         :param function_timeout : 超时秒数，函数运行超过这个时间，则自动杀死函数。为0是不限制。
-        :param threads_num:
-        :param specify_threadpool:使用指定的线程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
+        :param threads_num:线程或协程并发数量
+        :param specify_threadpool:使用指定的线程池/携程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
         :param concurrent_mode:并发模式，暂时支持 线程 、gevent、eventlet三种模式。  1线程  2 gevent 3 evenlet
         :param max_retry_times:
         :param log_level:
@@ -255,6 +263,9 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         :param logger_prefix: 日志前缀，可使不同的消费者生成不同的日志
         :param create_logger_file : 是否创建文件日志
         :param do_task_filtering :是否执行基于函数参数的任务过滤
+        :param task_filtering_expire_seconds:任务过滤的失效期，为0则永久性过滤任务。例如设置过滤过期时间是1800秒 ，
+               30分钟前发布过1 + 2 的任务，现在仍然执行，
+               如果是30分钟以内发布过这个任务，则不执行1 + 2，现在把这个逻辑集成到框架，一般用于接口价格缓存。
         :is_consuming_function_use_multi_params  函数的参数是否是传统的多参数，不为单个body字典表示多个参数。
         :param is_do_not_run_by_specify_time_effect :是否使不运行的时间段生效
         :param do_not_run_by_specify_time   :不运行的时间段
@@ -271,6 +282,17 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         concurrent_name = ConsumersManager.get_concurrent_name_by_concurrent_mode(concurrent_mode)
         current_queue__info_dict['concurrent_mode_name'] = concurrent_name
 
+        # 方便点击跳转定位到当前解释器下所有实例化消费者的文件行，点击可跳转到该处。
+        # 获取被调用函数在被调用时所处代码行数
+        # 直接实例化相应的类和使用工厂模式来实例化相应的类，得到的消费者实际实例化的行是不一样的，希望定位到用户的代码处，而不是定位到工厂模式处。
+        line = sys._getframe(0).f_back.f_lineno
+        # 获取被调用函数所在模块文件名
+        file_name = sys._getframe(1).f_code.co_filename
+        if 'consumer_factory.py' in file_name:
+            line = sys._getframe(1).f_back.f_lineno
+            file_name = sys._getframe(2).f_code.co_filename
+        current_queue__info_dict['where_to_instantiate'] = f'{file_name}:{line}'
+
         self._queue_name = queue_name
         self.queue_name = queue_name  # 可以换成公有的，免得外部访问有警告。
         self.consuming_function = consuming_function
@@ -298,11 +320,14 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         logger_name = f'{logger_prefix}{self.__class__.__name__}--{concurrent_name}--{queue_name}'
         # nb_print(logger_name)
         self.logger = LogManager(logger_name).get_logger_and_add_handlers(log_level, log_filename=f'{logger_name}.log' if create_logger_file else None)
-        self.logger.info(f'{self.__class__} 被实例化')
+        # self.logger.info(f'{self.__class__} 在 {current_queue__info_dict["where_to_instantiate"]}  被实例化')
+        sys.stdout.write(f'{time.strftime("%H:%M:%S")} "{current_queue__info_dict["where_to_instantiate"]}"  \033[0;30;44m此行 实例化队列名 {current_queue__info_dict["queue_name"]} 的消费者, 类型为 {self.__class__}\033[0m\n')
 
         self._do_task_filtering = do_task_filtering
-        self._redis_filter_key_name = f'filter:{queue_name}'
-        self._redis_filter = RedisFilter(self._redis_filter_key_name)
+
+        self._redis_filter_key_name = f'filter_zset:{queue_name}' if task_filtering_expire_seconds else f'filter_set:{queue_name}'
+        filter_class = RedisFilter if task_filtering_expire_seconds == 0 else RedisImpermanencyFilter
+        self._redis_filter = filter_class(self._redis_filter_key_name, task_filtering_expire_seconds)
 
         self._is_consuming_function_use_multi_params = is_consuming_function_use_multi_params
 
@@ -365,6 +390,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     def start_consuming_message(self):
         self.logger.warning(f'开始消费 {self._queue_name} 中的消息')
         self.threadpool.submit(self.keep_circulating(20)(self.check_heartbeat_and_message_count))
+        self._redis_filter.delete_expire_filter_task_cycle()
         if self._schedule_tasks_on_main_thread:
             self.keep_circulating(1)(self._shedual_task)()
         else:
