@@ -21,7 +21,7 @@ def get_consumer(queue_name, *, consuming_function: Callable = None, function_ti
                  specify_threadpool=None, concurrent_mode=1,
                  max_retry_times=3, log_level=10, is_print_detail_exception=True, msg_schedule_time_intercal=0.0,
                  qps=0, msg_expire_senconds=0,
-                 logger_prefix='', create_logger_file=True, do_task_filtering=False,
+                 logger_prefix='', create_logger_file=True, do_task_filtering=False, task_filtering_expire_seconds=0,
                  is_consuming_function_use_multi_params=True,
                  is_do_not_run_by_specify_time_effect=False, do_not_run_by_specify_time=('10:00:00', '22:00:00'),
                  schedule_tasks_on_main_thread=False,
@@ -47,6 +47,9 @@ def get_consumer(queue_name, *, consuming_function: Callable = None, function_ti
     :param logger_prefix: 日志前缀，可使不同的消费者生成不同的日志
     :param create_logger_file : 是否创建文件日志
     :param do_task_filtering :是否执行基于函数参数的任务过滤
+    :param task_filtering_expire_seconds:任务过滤的失效期，为0则永久性过滤任务。例如设置过滤过期时间是1800秒 ，
+           30分钟前发布过1 + 2 的任务，现在仍然执行，
+           如果是30分钟以内发布过这个任务，则不执行1 + 2，现在把这个逻辑集成到框架，一般用于接口价格缓存。
     :param is_consuming_function_use_multi_params  函数的参数是否是传统的多参数，不为单个body字典表示多个参数。
     :param is_do_not_run_by_specify_time_effect :是否使不运行的时间段生效
     :param do_not_run_by_specify_time   :不运行的时间段
