@@ -44,6 +44,11 @@ class RedisMixin(object):
     def redis_db8(self):
         return RedisManager(host=frame_config.REDIS_HOST, port=frame_config.REDIS_PORT, password=frame_config.REDIS_PASSWORD, db=8).get_redis()
 
+    @property
+    @decorators.cached_method_result
+    def redis_db6(self):
+        return RedisManager(host=frame_config.REDIS_HOST, port=frame_config.REDIS_PORT, password=frame_config.REDIS_PASSWORD, db=6).get_redis()
+
     @property
     @decorators.cached_method_result
     def redis_db_frame(self):
